#Created by R. David Murray (rdmurray@bitdance.com).
#This code is placed in the public domain.

__version__ = "$Id: zopectlTests.py,v 1.6 2001/12/10 03:00:27 rdmurray Exp $"

#For zopectl version 1.0.0

from unittest import TestCase, makeSuite, TextTestRunner
from os import remove, popen
from xfile import xfile
from sys import version
OLDZOPE = int(version[:1])<2

capturefn = 'zopectlTests.capture'
defaultfn = 'zopectlTests.defaults'
subparmfn = 'zopectlTests.parms'

#pyhomeinline = 'SOFTWARE_HOME = .'
pyhomeinline = 'PYTHONHOME = .'
if OLDZOPE: pyhomeoutline = 'PYTHONHOME=.'
else: pyhomeoutline = 'INST_HOME=.'

def capture(arglist):
	f = open(capturefn,'w')
	for arg in arglist:
		f.write("%s\n" % arg)
	f.close()

def makefile(fn,lines):
	f = open(fn,'w')
	for line in lines:
		f.write(line+'\n')
	f.close()


class TestCases(TestCase):

	def setUp(self):
		try: remove(capturefn)
		except OSError: pass
		try: remove(defaultfn)
		except OSError: pass
		try: remove(subparmfn)
		except OSError: pass
	tearDown = setUp

	def assertEQ(self,res,correct,msg=''):
		assert res==correct, "%s <> %s %s" % \
			(repr(res),repr(correct),msg)

	def assertSeqEQ(self,seq1,seq2):
		self.assertEQ(len(seq1),len(seq2))
		for i in range(max(len(seq1),len(seq2))):
			self.assertEQ(seq1[i] or '',seq2[i] or '',
				"(command output line %s)" % i)

	def assertCmdOutIs(self,args,out,debug=0):
		res = xfile(popen("../zopectl -n -f %s %s 2>&1" % \
				(defaultfn,args)))[:]
		if debug:
			for line in res: print line
		self.assertSeqEQ(res,out)

	def testNoArgs(self):
		self.assertCmdOutIs("",
			["zopectl: missing keyword (valid values are " +\
			"reload, restart, start, status and stop)"])

	def testNoConfFile(self):
		self.assertCmdOutIs("start",
			["zopectl: [Errno 2] No such file or directory: '%s'" \
				% defaultfn])

	def testEmptyConfFile(self):
		makefile(defaultfn,[])
		self.assertCmdOutIs("start",
			["zopectl: Missing value for SOFTWARE_HOME",
			 "zopectl: Skipping default"])

	def testPYTHONHOMEOnly(self):
		makefile(defaultfn,[pyhomeinline])
		self.assertCmdOutIs("start",
			[pyhomeoutline,
			 "python ./z2.py SITE=default",
			])

	def testPsuedoStartConfig(self):
		makefile(defaultfn,
			[pyhomeinline,
			 "DEBUG = on",
			])
		self.assertCmdOutIs("start",
			[pyhomeoutline,
			 "python ./z2.py -D SITE=default",
			])

	def testEscape(self):
		makefile(defaultfn,
			[pyhomeinline,
			 "PRODUCTS_PATH = %%(SOFTWARE_PRODUCTS)",
			])
		self.assertCmdOutIs("start",
			[pyhomeoutline,
			 "PRODUCTS_PATH=%(SOFTWARE_PRODUCTS)",
			 "python ./z2.py SITE=default",
			])

	def testNoSiteError(self):
		makefile(defaultfn,
			[pyhomeinline,
			 "SITECONF = %(SOFTWARE_HOME)/%(SITE).parms",
			])
		self.assertCmdOutIs("start",
			["zopectl: No sites specified.",
			])

	def testSimpleSubfile(self):
		makefile(defaultfn,
			[pyhomeinline,
			 "SITECONF = ./%(SITE).parms",
			 "BASE_PORT = 9000",
			])
		makefile(subparmfn,
			["BASE_PORT = 10000",
			])
		self.assertCmdOutIs("start zopectlTests",
			[pyhomeoutline,
			 "python ./z2.py -P 10000 SITE=zopectlTests",
			])

	def testSubfileWithEscape(self):
		makefile(defaultfn,
			[pyhomeinline,
			 "SITECONF = ./%(SITE).parms",
			 "PRODUCTS_PATH = %%(SOFTWARE_PRODUCTS)s",
			])
		makefile(subparmfn,
			["BASE_PORT = 10000",
			])
		self.assertCmdOutIs("start zopectlTests",
			[pyhomeoutline,
			 "PRODUCTS_PATH=%(SOFTWARE_PRODUCTS)s",
			 "python ./z2.py -P 10000 SITE=zopectlTests",
			])

	def testMoreComplexSubfile(self):
		makefile(defaultfn,
			[pyhomeinline,
			 "SITECONF = 	%(SOFTWARE_HOME)/%(SITE).parms",
			 "STUPID_LOG_FILE = %(SOFTWARE_HOME)/var/stupid.log",
			 "SERVICE_LIST = web webdav",
			 "SITELIST_all = zopectlTests",
			 "SITELIST_default = %(SITELIST_all)",
			])
		makefile(subparmfn,
			["BASE_PORT = 0",
			 "DEBUG = true",
			])
		self.assertCmdOutIs("start",
			[pyhomeoutline,
			 "STUPID_LOG_FILE=./var/stupid.log",
			 "python ./z2.py -D -w 80 -W 81 -f '' -m ''" +\
				" SITE=zopectlTests",
			])
		

suite = makeSuite(TestCases,'test')

if __name__ == '__main__':
	from sys import exit, argv
	if argv[1:]: capture(argv[1:]); exit(0)
	if not TextTestRunner().run(suite).wasSuccessful(): exit(1)
